@@ -15,13 +15,16 @@
 
 #![feature(box_patterns)]
 #![feature(conservative_impl_trait)]
+#![feature(fs_read_write)]
 #![feature(i128_type)]
+#![feature(libc)]
 #![feature(proc_macro_internals)]
 #![feature(quote)]
 #![feature(rustc_diagnostic_macros)]
 #![feature(specialization)]
 #![feature(rustc_private)]
 
+extern crate libc;
 #[macro_use]
 extern crate log;
 #[macro_use]
@@ -29,7 +32,6 @@ extern crate syntax;
 extern crate syntax_pos;
 extern crate flate2;
 extern crate serialize as rustc_serialize; // used by deriving
-extern crate owning_ref;
 extern crate rustc_errors as errors;
 extern crate syntax_ext;
 extern crate proc_macro;
@@ -54,6 +56,8 @@ mod link_args;
 
 pub mod creader;
 pub mod cstore;
+pub mod dynamic_lib;
 pub mod locator;
 
+#[cfg(not(stage0))] // remove after the next snapshot
 __build_diagnostic_array! { librustc_metadata, DIAGNOSTICS }
