@@ -12,57 +12,52 @@
 
 use super::metadata::{unknown_file_metadata, UNKNOWN_LINE_NUMBER};
 use super::utils::{DIB, debug_context};
+use monomorphize::Instance;
+use rustc::ty;
+use syntax::ast;
 
 use llvm;
 use llvm::debuginfo::DIScope;
 use rustc::hir::def_id::DefId;
 use rustc::hir::map::DefPathData;
-use common::CrateContext;
+use common::CodegenCx;
 
 use std::ffi::CString;
 use std::ptr;
 
-pub fn mangled_name_of_item(ccx: &CrateContext, def_id: DefId, extra: &str) -> String {
-    fn fill_nested(ccx: &CrateContext, def_id: DefId, extra: &str, output: &mut String) {
-        let def_key = ccx.tcx().def_key(def_id);
-        if let Some(parent) = def_key.parent {
-            fill_nested(ccx, DefId {
-                krate: def_id.krate,
-                index: parent
-            }, "", output);
-        }
-
-        let name = match def_key.disambiguated_data.data {
-            DefPathData::CrateRoot => ccx.tcx().crate_name(def_id.krate).as_str(),
-            data => data.as_interned_str()
-        };
-
-        output.push_str(&(name.len() + extra.len()).to_string());
-        output.push_str(&name);
-        output.push_str(extra);
-    }
-
-    let mut name = String::from("_ZN");
-    fill_nested(ccx, def_id, extra, &mut name);
-    name.push('E');
-    name
+pub fn mangled_name_of_instance<'a, 'tcx>(
+    cx: &CodegenCx<'a, 'tcx>,
+    instance: Instance<'tcx>,
+) -> ty::SymbolName {
+     let tcx = cx.tcx;
+     tcx.symbol_name(instance)
 }
 
-pub fn item_namespace(ccx: &CrateContext, def_id: DefId) -> DIScope {
-    if let Some(&scope) = debug_context(ccx).namespace_map.borrow().get(&def_id) {
+pub fn mangled_name_of_item<'a, 'tcx>(
+    cx: &CodegenCx<'a, 'tcx>,
+    node_id: ast::NodeId,
+) -> ty::SymbolName {
+    let tcx = cx.tcx;
+    let node_def_id = tcx.hir.local_def_id(node_id);
+    let instance = Instance::mono(tcx, node_def_id);
+    tcx.symbol_name(instance)
+}
+
+pub fn item_namespace(cx: &CodegenCx, def_id: DefId) -> DIScope {
+    if let Some(&scope) = debug_context(cx).namespace_map.borrow().get(&def_id) {
         return scope;
     }
 
-    let def_key = ccx.tcx().def_key(def_id);
+    let def_key = cx.tcx.def_key(def_id);
     let parent_scope = def_key.parent.map_or(ptr::null_mut(), |parent| {
-        item_namespace(ccx, DefId {
+        item_namespace(cx, DefId {
             krate: def_id.krate,
             index: parent
         })
     });
 
     let namespace_name = match def_key.disambiguated_data.data {
-        DefPathData::CrateRoot => ccx.tcx().crate_name(def_id.krate).as_str(),
+        DefPathData::CrateRoot => cx.tcx.crate_name(def_id.krate).as_str(),
         data => data.as_interned_str()
     };
 
@@ -70,13 +65,13 @@ pub fn item_namespace(ccx: &CrateContext, def_id: DefId) -> DIScope {
 
     let scope = unsafe {
         llvm::LLVMRustDIBuilderCreateNameSpace(
-            DIB(ccx),
+            DIB(cx),
             parent_scope,
             namespace_name.as_ptr(),
-            unknown_file_metadata(ccx),
+            unknown_file_metadata(cx),
             UNKNOWN_LINE_NUMBER)
     };
 
-    debug_context(ccx).namespace_map.borrow_mut().insert(def_id, scope);
+    debug_context(cx).namespace_map.borrow_mut().insert(def_id, scope);
     scope
 }
