@@ -10,12 +10,11 @@
 
 // aux-build:m1.rs
 
-#![feature(associated_consts)]
 
 extern crate m1;
 
 struct X {
 }
 
-impl m1::X for X {
+impl m1::X for X { //~ ERROR not all trait items implemented
 }
