@@ -10,19 +10,20 @@
 
 //! Types, helpers, and conversions to and from LSP and `racer` types.
 
-use std::collections::HashMap;
-use std::fmt::{self, Debug};
+use std::fmt;
 use std::path::PathBuf;
 use std::error::Error;
 
 use analysis::DefKind;
 use url::Url;
-use serde::Serialize;
 use span;
 use racer;
 use vfs::FileContents;
+use ls_types;
 
 pub use ls_types::*;
+pub use ls_types::request::Request as LSPRequest;
+pub use ls_types::notification::Notification as LSPNotification;
 
 /// Errors that can occur when parsing a file URI.
 #[derive(Debug)]
@@ -53,21 +54,17 @@ where
 
 /// Parse the given URI into a `PathBuf`.
 pub fn parse_file_path(uri: &Url) -> Result<PathBuf, UrlFileParseError> {
-    if uri.scheme() != "file" {
-        Err(UrlFileParseError::InvalidScheme)
-    } else {
+    if uri.scheme() == "file" {
         uri.to_file_path()
             .map_err(|_err| UrlFileParseError::InvalidFilePath)
+    } else {
+        Err(UrlFileParseError::InvalidScheme)
     }
 }
 
 /// Create an edit for the given location and text.
 pub fn make_workspace_edit(location: Location, new_text: String) -> WorkspaceEdit {
-    let mut edit = WorkspaceEdit {
-        changes: HashMap::new(),
-    };
-
-    edit.changes.insert(
+    let changes = vec![(
         location.uri,
         vec![
             TextEdit {
@@ -75,9 +72,12 @@ pub fn make_workspace_edit(location: Location, new_text: String) -> WorkspaceEdi
                 new_text,
             },
         ],
-    );
+    )].into_iter().collect();
 
-    edit
+    WorkspaceEdit {
+        changes: Some(changes),
+        document_changes: None,
+    }
 }
 
 /// Utilities for working with the language server protocol.
@@ -103,7 +103,7 @@ pub mod ls_util {
 
     /// Convert a language server protocol location into an RLS span.
     pub fn location_to_rls(
-        l: Location,
+        l: &Location,
     ) -> Result<span::Span<span::ZeroIndexed>, UrlFileParseError> {
         parse_file_path(&l.uri).map(|path| Span::from_range(range_to_rls(l.range), path))
     }
@@ -136,8 +136,8 @@ pub mod ls_util {
     /// Convert an RLS position into a language server protocol range.
     pub fn rls_to_position(p: span::Position<span::ZeroIndexed>) -> Position {
         Position {
-            line: p.row.0 as u64,
-            character: p.col.0 as u64,
+            line: p.row.0.into(),
+            character: p.col.0.into(),
         }
     }
 
@@ -181,17 +181,21 @@ pub mod ls_util {
 /// Convert an RLS def-kind to a language server protocol symbol-kind.
 pub fn source_kind_from_def_kind(k: DefKind) -> SymbolKind {
     match k {
-        DefKind::Enum => SymbolKind::Enum,
-        DefKind::TupleVariant => SymbolKind::Constant,
-        DefKind::StructVariant => SymbolKind::Class,
+        DefKind::Enum | DefKind::Union => SymbolKind::Enum,
+        DefKind::Static | DefKind::Const => SymbolKind::Constant,
         DefKind::Tuple => SymbolKind::Array,
         DefKind::Struct => SymbolKind::Class,
-        DefKind::Union => SymbolKind::Class,
-        DefKind::Trait => SymbolKind::Interface,
-        DefKind::Function | DefKind::Method | DefKind::Macro => SymbolKind::Function,
+        DefKind::Function | DefKind::Macro => SymbolKind::Function,
+        DefKind::Method => SymbolKind::Method,
         DefKind::Mod => SymbolKind::Module,
-        DefKind::Type | DefKind::ExternType => SymbolKind::Interface,
-        DefKind::Local | DefKind::Static | DefKind::Const | DefKind::Field => SymbolKind::Variable,
+        DefKind::Trait | DefKind::Type | DefKind::ExternType => SymbolKind::Interface,
+        DefKind::Local => SymbolKind::Variable,
+        DefKind::Field | DefKind::TupleVariant | DefKind::StructVariant => SymbolKind::Field,
+
+        // Waiting for languageserver-types be updated to LSP 3 spec
+        //DefKind::Struct => SymbolKind::Struct,
+        //DefKind::TupleVariant | DefKind::StructVariant => SymbolKind::EnumMember,
+        //DefKind::Type => SymbolKind::TypeParameter,
     }
 }
 
@@ -221,7 +225,7 @@ pub fn completion_kind_from_match_type(m: racer::MatchType) -> CompletionItemKin
 }
 
 /// Convert a racer match into an RLS completion.
-pub fn completion_item_from_racer_match(m: racer::Match) -> CompletionItem {
+pub fn completion_item_from_racer_match(m: &racer::Match) -> CompletionItem {
     let mut item = CompletionItem::new_simple(m.matchstr.clone(), m.contextstr.clone());
     item.kind = Some(completion_kind_from_match_type(m.mtype));
 
@@ -262,32 +266,123 @@ impl Default for InitializationOptions {
     }
 }
 
-/// A JSON language server protocol request that will have a matching response.
-#[derive(Debug, Serialize)]
-pub struct RequestMessage<T>
-where
-    T: Debug + Serialize,
-{
-    jsonrpc: &'static str,
-    /// The request id. The response will have a matching id.
-    pub id: u32,
-    /// The well-known language server protocol request method string.
-    pub method: String,
-    /// Extra request parameters.
-    pub params: T,
+// Subset of flags from ls_types::ClientCapabilities that affects this RLS.
+// Passed in the `initialize` request under `capabilities`.
+#[derive(Debug, PartialEq, Deserialize, Serialize, Clone, Copy, Default)]
+#[serde(default)]
+pub struct ClientCapabilities {
+    pub code_completion_has_snippet_support: bool,
 }
 
-impl<T> RequestMessage<T>
-where
-    T: Debug + Serialize,
-{
-    /// Construct a new request.
-    pub fn new(id: u32, method: String, params: T) -> Self {
-        RequestMessage {
-            jsonrpc: "2.0",
-            id,
-            method: method,
-            params: params,
+impl ClientCapabilities {
+    pub fn new(params: &ls_types::InitializeParams) -> ClientCapabilities {
+        // ls_types::ClientCapabilities is a rather awkward object to use internally
+        // (for instance it doesn't Clone). Instead we pick out the bits of it that we
+        // are going to handle into ClientCapabilities. The upside of
+        // using this very simple struct is that it can be kept thread safe
+        // without mutex locking it on every request.
+        let code_completion_has_snippet_support = params
+        .capabilities
+        .text_document
+        .as_ref()
+        .and_then(|doc| doc.completion.as_ref())
+        .and_then(|comp| comp.completion_item.as_ref())
+        .and_then(|item| item.snippet_support.as_ref())
+        .unwrap_or(&false)
+        .to_owned();
+
+        ClientCapabilities {
+            code_completion_has_snippet_support,
         }
     }
 }
+
+/* --------------- Custom JSON-RPC notifications -------------- */
+
+/// Custom LSP notification sent to client indicating that the server is currently
+/// processing data and may publish new diagnostics on `rustDocument/diagnosticsEnd`.
+#[derive(Debug)]
+pub enum DiagnosticsBegin { }
+
+impl LSPNotification for DiagnosticsBegin {
+    type Params = ();
+    const METHOD: &'static str = "rustDocument/diagnosticsBegin";
+}
+
+/// Custom LSP notification sent to client indicating that data processing started
+/// by a `rustDocument`/diagnosticsBegin` has ended.
+/// For each `diagnosticsBegin` message, there is a single `diagnosticsEnd` message.
+/// This means that for multiple active `diagnosticsBegin` messages, there will
+/// be sent multiple `diagnosticsEnd` notifications.
+#[derive(Debug)]
+pub enum DiagnosticsEnd { }
+
+impl LSPNotification for DiagnosticsEnd {
+    type Params = ();
+    const METHOD: &'static str = "rustDocument/diagnosticsEnd";
+}
+
+/// Custom LSP notification sent to client indicating that a build process has begun.
+#[derive(Debug)]
+pub enum BeginBuild { }
+
+impl LSPNotification for BeginBuild {
+    type Params = ();
+    const METHOD: &'static str = "rustDocument/beginBuild";
+}
+
+/* ------------------ Custom JSON-RPC requests ---------------- */
+
+/// Find all the implementations of a given trait.
+#[derive(Debug)]
+pub enum FindImpls { }
+
+impl LSPRequest for FindImpls {
+    type Params = TextDocumentPositionParams;
+    type Result = Vec<Location>;
+    const METHOD: &'static str = "rustDocument/implementations";
+}
+
+/* ----------  Temporary LSP type until window/progress proposal is done --------- */
+
+// Notification from server to client for build progress.
+#[derive(Debug)]
+pub struct Progress;
+
+impl notification::Notification for Progress {
+    type Params = ProgressParams;
+    const METHOD: &'static str = NOTIFICATION__Progress;
+}
+
+/**
+ * The progress notification is sent from the server to the client to ask the client
+ * to indicate progress.
+ */
+#[allow(non_upper_case_globals)]
+pub const NOTIFICATION__Progress: &str = "window/progress";
+
+#[derive(Debug, PartialEq, Deserialize, Serialize, Clone)]
+pub struct ProgressParams {
+    // A unique identifier to associate multiple progress notifications with the same progress.
+    pub id: String,
+
+    // The title of the progress.
+    // This should be the same for all ProgressParams with the same id.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub title: Option<String>,
+
+    // Optional progress message to display.
+    // If unset, the previous progress message (if any) is still valid.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub message: Option<String>,
+
+    // Optional progress percentage to display.
+    // If unset, the previous progress percentage (if any) is still valid.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub percentage: Option<f64>,
+
+    // Set to true on the final progress update.
+    // No more progress notifications with the same ID should be sent.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub done: Option<bool>,
+}
