@@ -17,9 +17,9 @@ use std::fs::File;
 use std::io::{Error, ErrorKind, Read};
 use std::path::{Path, PathBuf};
 
+use Summary;
 use file_lines::FileLines;
 use lists::{ListTactic, SeparatorPlace, SeparatorTactic};
-use Summary;
 
 /// Check if we're in a nightly build.
 ///
@@ -518,6 +518,7 @@ macro_rules! create_config {
 
             pub fn print_docs() {
                 use std::cmp;
+                const HIDE_OPTIONS: [&str; 3] = ["verbose", "file_lines", "width_heuristics"];
                 let max = 0;
                 $( let max = cmp::max(max, stringify!($i).len()+1); )+
                 let mut space_str = String::with_capacity(max);
@@ -527,20 +528,23 @@ macro_rules! create_config {
                 println!("Configuration Options:");
                 $(
                     let name_raw = stringify!($i);
-                    let mut name_out = String::with_capacity(max);
-                    for _ in name_raw.len()..max-1 {
-                        name_out.push(' ')
+
+                    if !HIDE_OPTIONS.contains(&name_raw) {
+                        let mut name_out = String::with_capacity(max);
+                        for _ in name_raw.len()..max-1 {
+                            name_out.push(' ')
+                        }
+                        name_out.push_str(name_raw);
+                        name_out.push(' ');
+                        println!("{}{} Default: {:?}",
+                                name_out,
+                                <$ty>::doc_hint(),
+                                $def);
+                        $(
+                            println!("{}{}", space_str, $dstring);
+                        )+
+                        println!();
                     }
-                    name_out.push_str(name_raw);
-                    name_out.push(' ');
-                    println!("{}{} Default: {:?}",
-                             name_out,
-                             <$ty>::doc_hint(),
-                             $def);
-                    $(
-                        println!("{}{}", space_str, $dstring);
-                    )+
-                    println!();
                 )+
             }
 
@@ -626,6 +630,7 @@ create_config! {
     reorder_imports_in_group: bool, false, false, "Reorder import statements in group";
     reorder_imported_names: bool, true, false,
         "Reorder lists of names in import statements alphabetically";
+    reorder_modules: bool, false, false, "Reorder module statemtents alphabetically in group";
 
     // Spaces around punctuation
     binop_separator: SeparatorPlace, SeparatorPlace::Front, false,
@@ -644,8 +649,6 @@ create_config! {
                                              threshold.";
     remove_blank_lines_at_start_or_end_of_block: bool, true, false,
         "Remove blank lines at start or end of a block";
-    same_line_attributes: bool, true, false,
-        "Try to put attributes on the same line as fields and variants.";
     match_arm_blocks: bool, true, false, "Wrap the body of arms in blocks when it does not fit on \
         the same line with the pattern of arms";
     force_multiline_blocks: bool, false, false,
@@ -671,6 +674,7 @@ create_config! {
     condense_wildcard_suffixes: bool, false, false, "Replace strings of _ wildcards by a single .. \
                                               in tuple patterns";
     force_explicit_abi: bool, true, true, "Always print the abi for extern items";
+    use_field_init_shorthand: bool, false, false, "Use field initialization shorthand if possible";
 
     // Control options (changes the operation of rustfmt, rather than the formatting)
     write_mode: WriteMode, WriteMode::Overwrite, false,
@@ -684,6 +688,7 @@ create_config! {
             "Enables unstable features. Only available on nightly channel";
     disable_all_formatting: bool, false, false, "Don't reformat anything";
     skip_children: bool, false, false, "Don't reformat out of line modules";
+    hide_parse_errors: bool, false, false, "Hide errors from the parser";
     error_on_line_overflow: bool, true, false, "Error if unable to get all lines within max_width";
     error_on_unformatted: bool, false, false,
         "Error if unable to get comments or string literals within max_width, \
