@@ -178,7 +178,7 @@ pub fn last_line_extendable(s: &str) -> bool {
     }
     for c in s.chars().rev() {
         match c {
-            ')' | ']' | '}' | '?' => continue,
+            ')' | ']' | '}' | '?' | '>' => continue,
             '\n' => break,
             _ if c.is_whitespace() => continue,
             _ => return false,
@@ -213,19 +213,6 @@ pub fn contains_skip(attrs: &[Attribute]) -> bool {
         .any(|a| a.meta().map_or(false, |a| is_skip(&a)))
 }
 
-// Find the end of a TyParam
-#[inline]
-pub fn end_typaram(typaram: &ast::TyParam) -> BytePos {
-    typaram
-        .bounds
-        .last()
-        .map_or(typaram.span, |bound| match *bound {
-            ast::RegionTyParamBound(ref lt) => lt.span,
-            ast::TraitTyParamBound(ref prt, _) => prt.span,
-        })
-        .hi()
-}
-
 #[inline]
 pub fn semicolon_for_expr(context: &RewriteContext, expr: &ast::Expr) -> bool {
     match expr.node {
@@ -267,17 +254,6 @@ pub fn count_newlines(input: &str) -> usize {
     input.chars().filter(|&c| c == '\n').count()
 }
 
-#[inline]
-pub fn trim_newlines(input: &str) -> &str {
-    match input.find(|c| c != '\n' && c != '\r') {
-        Some(start) => {
-            let end = input.rfind(|c| c != '\n' && c != '\r').unwrap_or(0) + 1;
-            &input[start..end]
-        }
-        None => "",
-    }
-}
-
 // Macro for deriving implementations of Serialize/Deserialize for enums
 #[macro_export]
 macro_rules! impl_enum_serialize_and_deserialize {
