@@ -15,23 +15,47 @@ use syntax::attr::HasAttrs;
 use syntax::codemap::{self, BytePos, CodeMap, Pos, Span};
 use syntax::parse::ParseSess;
 
-use expr::rewrite_literal;
-use spanned::Spanned;
 use codemap::{LineRangeUtils, SpanUtils};
-use comment::{combine_strs_with_missing_comments, contains_comment, remove_trailing_white_spaces,
-              CodeCharKind, CommentCodeSlices, FindUncommented};
+use comment::{combine_strs_with_missing_comments, contains_comment, CodeCharKind,
+              CommentCodeSlices, FindUncommented};
 use comment::rewrite_comment;
 use config::{BraceStyle, Config};
+use expr::rewrite_literal;
 use items::{format_impl, format_trait, format_trait_alias, rewrite_associated_impl_type,
             rewrite_associated_type, rewrite_type_alias, FnSig, StaticParts, StructParts};
 use lists::{itemize_list, write_list, DefinitiveListTactic, ListFormatting, SeparatorPlace,
             SeparatorTactic};
-use macros::{rewrite_macro, MacroPosition};
+use macros::{rewrite_macro, rewrite_macro_def, MacroPosition};
 use regex::Regex;
 use rewrite::{Rewrite, RewriteContext};
 use shape::{Indent, Shape};
+use spanned::Spanned;
 use utils::{self, contains_skip, count_newlines, inner_attributes, mk_sp, ptr_vec_to_ref_vec};
 
+/// Returns attributes that are within `outer_span`.
+pub fn filter_inline_attrs(attrs: &[ast::Attribute], outer_span: Span) -> Vec<ast::Attribute> {
+    attrs
+        .iter()
+        .filter(|a| outer_span.lo() <= a.span.lo() && a.span.hi() <= outer_span.hi())
+        .cloned()
+        .collect()
+}
+
+/// Returns true for `mod foo;`, false for `mod foo { .. }`.
+fn is_mod_decl(item: &ast::Item) -> bool {
+    match item.node {
+        ast::ItemKind::Mod(ref m) => m.inner.hi() != item.span.hi(),
+        _ => false,
+    }
+}
+
+/// Returns true for `mod foo;` without any inline attributes.
+/// We cannot reorder modules with attributes because doing so can break the code.
+/// e.g. `#[macro_use]`.
+fn is_mod_decl_without_attr(item: &ast::Item) -> bool {
+    is_mod_decl(item) && filter_inline_attrs(&item.attrs, item.span()).is_empty()
+}
+
 fn is_use_item(item: &ast::Item) -> bool {
     match item.node {
         ast::ItemKind::Use(_) => true,
@@ -318,39 +342,26 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
         let filtered_attrs;
         let mut attrs = &item.attrs;
         match item.node {
-            ast::ItemKind::Mod(ref m) => {
-                let outer_file = self.codemap.lookup_char_pos(item.span.lo()).file;
-                let inner_file = self.codemap.lookup_char_pos(m.inner.lo()).file;
-                if outer_file.name == inner_file.name {
-                    // Module is inline, in this case we treat modules like any
-                    // other item.
-                    if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
-                        self.push_skipped_with_span(item.span());
-                        return;
-                    }
-                } else if contains_skip(&item.attrs) {
-                    // Module is not inline, but should be skipped.
+            // Module is inline, in this case we treat it like any other item.
+            _ if !is_mod_decl(item) => {
+                if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
+                    self.push_skipped_with_span(item.span());
                     return;
-                } else {
-                    // Module is not inline and should not be skipped. We want
-                    // to process only the attributes in the current file.
-                    filtered_attrs = item.attrs
-                        .iter()
-                        .filter_map(|a| {
-                            let attr_file = self.codemap.lookup_char_pos(a.span.lo()).file;
-                            if attr_file.name == outer_file.name {
-                                Some(a.clone())
-                            } else {
-                                None
-                            }
-                        })
-                        .collect::<Vec<_>>();
-                    // Assert because if we should skip it should be caught by
-                    // the above case.
-                    assert!(!self.visit_attrs(&filtered_attrs, ast::AttrStyle::Outer));
-                    attrs = &filtered_attrs;
                 }
             }
+            // Module is not inline, but should be skipped.
+            ast::ItemKind::Mod(..) if contains_skip(&item.attrs) => {
+                return;
+            }
+            // Module is not inline and should not be skipped. We want
+            // to process only the attributes in the current file.
+            ast::ItemKind::Mod(..) => {
+                filtered_attrs = filter_inline_attrs(&item.attrs, item.span());
+                // Assert because if we should skip it should be caught by
+                // the above case.
+                assert!(!self.visit_attrs(&filtered_attrs, ast::AttrStyle::Outer));
+                attrs = &filtered_attrs;
+            }
             _ => {
                 if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
                     self.push_skipped_with_span(item.span());
@@ -397,8 +408,9 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
                 self.last_pos = source!(self, item.span).hi();
             }
             ast::ItemKind::Mod(ref module) => {
+                let is_inline = !is_mod_decl(item);
                 self.format_missing_with_indent(source!(self, item.span).lo());
-                self.format_mod(module, &item.vis, item.span, item.ident, attrs);
+                self.format_mod(module, &item.vis, item.span, item.ident, attrs, is_inline);
             }
             ast::ItemKind::Mac(ref mac) => {
                 self.visit_mac(mac, Some(item.ident), MacroPosition::Item);
@@ -410,9 +422,6 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
             ast::ItemKind::Static(..) | ast::ItemKind::Const(..) => {
                 self.visit_static(&StaticParts::from_item(item));
             }
-            ast::ItemKind::AutoImpl(..) => {
-                // FIXME(#78): format impl definitions.
-            }
             ast::ItemKind::Fn(ref decl, unsafety, constness, abi, ref generics, ref body) => {
                 self.visit_fn(
                     visit::FnKind::ItemFn(item.ident, unsafety, constness, abi, &item.vis, body),
@@ -439,10 +448,16 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
                 let snippet = Some(self.snippet(item.span).to_owned());
                 self.push_rewrite(item.span, snippet);
             }
-            ast::ItemKind::MacroDef(..) => {
-                // FIXME(#1539): macros 2.0
-                let mac_snippet = Some(remove_trailing_white_spaces(self.snippet(item.span)));
-                self.push_rewrite(item.span, mac_snippet);
+            ast::ItemKind::MacroDef(ref def) => {
+                let rewrite = rewrite_macro_def(
+                    &self.get_context(),
+                    self.block_indent,
+                    def,
+                    item.ident,
+                    &item.vis,
+                    item.span,
+                );
+                self.push_rewrite(item.span, rewrite);
             }
         }
     }
@@ -573,14 +588,14 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
         snippet_provider: &'a SnippetProvider,
     ) -> FmtVisitor<'a> {
         FmtVisitor {
-            parse_session: parse_session,
+            parse_session,
             codemap: parse_session.codemap(),
             buffer: String::with_capacity(snippet_provider.big_snippet.len() * 2),
             last_pos: BytePos(0),
             block_indent: Indent::empty(),
-            config: config,
+            config,
             is_if_else_block: false,
-            snippet_provider: snippet_provider,
+            snippet_provider,
             line_number: 0,
             skipped_range: vec![],
         }
@@ -646,34 +661,35 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
     }
 
     fn walk_items(&mut self, mut items_left: &[&ast::Item]) {
-        while !items_left.is_empty() {
-            // If the next item is a `use` declaration, then extract it and any subsequent `use`s
-            // to be potentially reordered within `format_imports`. Otherwise, just format the
-            // next item for output.
-            if self.config.reorder_imports() && is_use_item(&*items_left[0]) {
-                let used_items_len = self.reorder_items(
-                    items_left,
-                    &is_use_item,
-                    self.config.reorder_imports_in_group(),
-                );
+        macro try_reorder_items_with($reorder: ident, $in_group: ident, $pred: ident) {
+            if self.config.$reorder() && $pred(&*items_left[0]) {
+                let used_items_len =
+                    self.reorder_items(items_left, &$pred, self.config.$in_group());
                 let (_, rest) = items_left.split_at(used_items_len);
                 items_left = rest;
-            } else if self.config.reorder_extern_crates() && is_extern_crate(&*items_left[0]) {
-                let used_items_len = self.reorder_items(
-                    items_left,
-                    &is_extern_crate,
-                    self.config.reorder_extern_crates_in_group(),
-                );
-                let (_, rest) = items_left.split_at(used_items_len);
-                items_left = rest;
-            } else {
-                // `unwrap()` is safe here because we know `items_left`
-                // has elements from the loop condition
-                let (item, rest) = items_left.split_first().unwrap();
-                self.visit_item(item);
-                items_left = rest;
+                continue;
             }
         }
+
+        while !items_left.is_empty() {
+            // If the next item is a `use`, `extern crate` or `mod`, then extract it and any
+            // subsequent items that have the same item kind to be reordered within
+            // `format_imports`. Otherwise, just format the next item for output.
+            {
+                try_reorder_items_with!(reorder_imports, reorder_imports_in_group, is_use_item);
+                try_reorder_items_with!(
+                    reorder_extern_crates,
+                    reorder_extern_crates_in_group,
+                    is_extern_crate
+                );
+                try_reorder_items_with!(reorder_modules, reorder_modules, is_mod_decl_without_attr);
+            }
+            // Reaching here means items were not reordered. There must be at least
+            // one item left in `items_left`, so calling `unwrap()` here is safe.
+            let (item, rest) = items_left.split_first().unwrap();
+            self.visit_item(item);
+            items_left = rest;
+        }
     }
 
     fn walk_mod_items(&mut self, m: &ast::Mod) {
@@ -719,13 +735,8 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
         s: Span,
         ident: ast::Ident,
         attrs: &[ast::Attribute],
+        is_internal: bool,
     ) {
-        // Decide whether this is an inline mod or an external mod.
-        let local_file_name = self.codemap.span_to_filename(s);
-        let inner_span = source!(self, m.inner);
-        let is_internal = !(inner_span.lo().0 == 0 && inner_span.hi().0 == 0)
-            && local_file_name == self.codemap.span_to_filename(inner_span);
-
         self.push_str(&*utils::format_visibility(vis));
         self.push_str("mod ");
         self.push_str(&ident.to_string());
@@ -1050,27 +1061,16 @@ fn format_derive(context: &RewriteContext, derive_args: &[&str], shape: Shape) -
 }
 
 fn is_derive(attr: &ast::Attribute) -> bool {
-    match attr.meta() {
-        Some(meta_item) => match meta_item.node {
-            ast::MetaItemKind::List(..) => meta_item.name.as_str() == "derive",
-            _ => false,
-        },
-        _ => false,
-    }
+    attr.check_name("derive")
 }
 
 /// Returns the arguments of `#[derive(...)]`.
 fn get_derive_args<'a>(context: &'a RewriteContext, attr: &ast::Attribute) -> Option<Vec<&'a str>> {
-    attr.meta().and_then(|meta_item| match meta_item.node {
-        ast::MetaItemKind::List(ref args) if meta_item.name.as_str() == "derive" => {
-            // Every argument of `derive` should be `NestedMetaItemKind::Literal`.
-            Some(
-                args.iter()
-                    .map(|a| context.snippet(a.span))
-                    .collect::<Vec<_>>(),
-            )
-        }
-        _ => None,
+    attr.meta_item_list().map(|meta_item_list| {
+        meta_item_list
+            .iter()
+            .map(|nested_meta_item| context.snippet(nested_meta_item.span))
+            .collect()
     })
 }
 
