@@ -100,6 +100,16 @@ trait Foo {
         <MergingUntypedTimeSeries<SliceSeries<SliceWindow>> as UntypedTimeSeries>::IterRev;
 }
 
+// #2331
+trait MyTrait<
+    AAAAAAAAAAAAAAAAAAAA,
+    BBBBBBBBBBBBBBBBBBBB,
+    CCCCCCCCCCCCCCCCCCCC,
+    DDDDDDDDDDDDDDDDDDDD,
+> {
+    fn foo() {}
+}
+
 // Trait aliases
 trait FooBar = Foo + Bar;
 trait FooBar<A, B, C> = Foo + Bar;
